---
title: llarp::layers::platform::PlatformDriver
summary: base type that is responsible for bridging the os and the flow layer. 

---

# llarp::layers::platform::PlatformDriver



base type that is responsible for bridging the os and the flow layer.  [More...](#detailed-description)


`#include <platform_driver.hpp>`

## Public Functions

|                | Name           |
| -------------- | -------------- |
| | **[PlatformDriver](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#function-platformdriver)**(const [PlatformDriver](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md) & ) =delete |
| | **[PlatformDriver](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#function-platformdriver)**([PlatformDriver](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md) && ) =delete |
| | **[PlatformDriver](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#function-platformdriver)**([PlatformDriver_pimpl](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver__pimpl.md) * impl, [flow::FlowLayer](Classes/classllarp_1_1layers_1_1flow_1_1FlowLayer.md) & flow_layer, std::shared_ptr< [NetworkConfig](Classes/structllarp_1_1NetworkConfig.md) > netconf, std::shared_ptr< [DnsConfig](Classes/structllarp_1_1DnsConfig.md) > dnsconf) |
| virtual void | **[got_os_traffic](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#function-got-os-traffic)**(std::vector< [OSTraffic](Classes/structllarp_1_1layers_1_1platform_1_1OSTraffic.md) > pkts)<br>called each time we get a bunch of os traffic.  |
| std::shared_ptr< [NameResolverJob_Base](Classes/classllarp_1_1layers_1_1platform_1_1NameResolverJob__Base.md) > | **[make_resolver_job_for](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#function-make-resolver-job-for)**(UnresolvedName_t name) const<br>given a name we want to look up, make a one shot job that we will use to look it up from the network.  |
| void | **[map_remote](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#function-map-remote)**(std::string name, std::string auth, std::vector< [IPRange](Classes/structllarp_1_1IPRange.md) > ranges, std::function< void(std::optional< [flow::FlowInfo](Classes/structllarp_1_1layers_1_1flow_1_1FlowInfo.md) >, std::string)> result_handler) |
| void | **[map_exit](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#function-map-exit)**(std::string name, std::string auth, std::vector< [IPRange](Classes/structllarp_1_1IPRange.md) > ranges, std::function< void(bool, std::string)> result_handler)<br>calls map_remote() for exits  |
| [PlatformStats](Classes/structllarp_1_1layers_1_1platform_1_1PlatformStats.md) | **[current_stats](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#function-current-stats)**() const |

## Protected Functions

|                | Name           |
| -------------- | -------------- |
| void | **[on_before_loop_ready](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#function-on-before-loop-ready)**()<br>called once per event loop tick, before polling for io.  |
| bool | **[on_loop_ready](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#function-on-loop-ready)**()<br>called once per event loop tick after platform layer event loop is ready to be read.  |
| void | **[after_loop_ready](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#function-after-loop-ready)**()<br>called once per event loop after we have processed all our events from a read cycle.  |

## Protected Attributes

|                | Name           |
| -------------- | -------------- |
| std::unique_ptr< [PlatformDriver_pimpl](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver__pimpl.md) > | **[_impl](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#variable--impl)**  |
| [flow::FlowLayer](Classes/classllarp_1_1layers_1_1flow_1_1FlowLayer.md) & | **[_flow_layer](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#variable--flow-layer)**  |
| std::shared_ptr< [EventLoopWakeup](Classes/classllarp_1_1EventLoopWakeup.md) > | **[_os_send](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#variable--os-send)** <br>called to wake up the loop after we sent things out on the network  |
| std::shared_ptr< [EventLoopWakeup](Classes/classllarp_1_1EventLoopWakeup.md) > | **[_os_recv](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#variable--os-recv)** <br>called to wake up the loop to read os traffic  |
| [OSTraffic_IO_Base](Classes/classllarp_1_1layers_1_1platform_1_1OSTraffic__IO__Base.md) | **[_os_io](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#variable--os-io)**  |
| [AddrMapper](Classes/classllarp_1_1layers_1_1platform_1_1AddrMapper.md) | **[_addr_mapper](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#variable--addr-mapper)**  |
| std::shared_ptr< [llarp::dns::Server](Classes/classllarp_1_1dns_1_1Server.md) > | **[_dns](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#variable--dns)**  |

## Friends

|                | Name           |
| -------------- | -------------- |
| class | **[OSTraffic_IO_Base](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#friend-ostraffic-io-base)**  |

## Detailed Description

```cpp
class llarp::layers::platform::PlatformDriver;
```

base type that is responsible for bridging the os and the flow layer. 

each kind of ethertype has its own implementation. 

## Public Functions Documentation

### function PlatformDriver

```cpp
PlatformDriver(
    const PlatformDriver & 
) =delete
```


### function PlatformDriver

```cpp
PlatformDriver(
    PlatformDriver && 
) =delete
```


### function PlatformDriver

```cpp
PlatformDriver(
    PlatformDriver_pimpl * impl,
    flow::FlowLayer & flow_layer,
    std::shared_ptr< NetworkConfig > netconf,
    std::shared_ptr< DnsConfig > dnsconf
)
```


### function got_os_traffic

```cpp
virtual void got_os_traffic(
    std::vector< OSTraffic > pkts
)
```

called each time we get a bunch of os traffic. 

overrided by each kind of platform 


### function make_resolver_job_for

```cpp
std::shared_ptr< NameResolverJob_Base > make_resolver_job_for(
    UnresolvedName_t name
) const
```

given a name we want to look up, make a one shot job that we will use to look it up from the network. 

### function map_remote

```cpp
void map_remote(
    std::string name,
    std::string auth,
    std::vector< IPRange > ranges,
    std::function< void(std::optional< flow::FlowInfo >, std::string)> result_handler
)
```


### function map_exit

```cpp
void map_exit(
    std::string name,
    std::string auth,
    std::vector< IPRange > ranges,
    std::function< void(bool, std::string)> result_handler
)
```

calls map_remote() for exits 

### function current_stats

```cpp
PlatformStats current_stats() const
```


## Protected Functions Documentation

### function on_before_loop_ready

```cpp
void on_before_loop_ready()
```

called once per event loop tick, before polling for io. 

### function on_loop_ready

```cpp
bool on_loop_ready()
```

called once per event loop tick after platform layer event loop is ready to be read. 

returns true if we proccessed events and thus need to call [after_loop_ready()](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md#function-after-loop-ready). 


### function after_loop_ready

```cpp
void after_loop_ready()
```

called once per event loop after we have processed all our events from a read cycle. 

## Protected Attributes Documentation

### variable _impl

```cpp
std::unique_ptr< PlatformDriver_pimpl > _impl;
```


### variable _flow_layer

```cpp
flow::FlowLayer & _flow_layer;
```


### variable _os_send

```cpp
std::shared_ptr< EventLoopWakeup > _os_send;
```

called to wake up the loop after we sent things out on the network 

### variable _os_recv

```cpp
std::shared_ptr< EventLoopWakeup > _os_recv;
```

called to wake up the loop to read os traffic 

### variable _os_io

```cpp
OSTraffic_IO_Base _os_io;
```


### variable _addr_mapper

```cpp
AddrMapper _addr_mapper;
```


### variable _dns

```cpp
std::shared_ptr< llarp::dns::Server > _dns;
```


## Friends

### friend OSTraffic_IO_Base

```cpp
friend class OSTraffic_IO_Base(
    OSTraffic_IO_Base 
);
```


-------------------------------

Updated on 2023-02-01 at 19:28:29 +0000