---
title: llarp/layers/platform/platform_driver.hpp

---

# llarp/layers/platform/platform_driver.hpp



## Namespaces

| Name           |
| -------------- |
| **[llarp](Namespaces/namespacellarp.md)** <br>[crypto.hpp] |
| **[llarp::dns](Namespaces/namespacellarp_1_1dns.md)**  |
| **[llarp::vpn](Namespaces/namespacellarp_1_1vpn.md)**  |
| **[llarp::layers](Namespaces/namespacellarp_1_1layers.md)**  |
| **[llarp::layers::flow](Namespaces/namespacellarp_1_1layers_1_1flow.md)**  |
| **[llarp::layers::platform](Namespaces/namespacellarp_1_1layers_1_1platform.md)**  |

## Classes

|                | Name           |
| -------------- | -------------- |
| class | **[llarp::layers::platform::OSTraffic_IO_Base](Classes/classllarp_1_1layers_1_1platform_1_1OSTraffic__IO__Base.md)**  |
| class | **[llarp::layers::platform::PlatformDriver](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md)** <br>base type that is responsible for bridging the os and the flow layer.  |




## Source code

```cpp
#pragma once

#include "addr_mapper.hpp"
#include "llarp/layers/platform/platform_addr.hpp"
#include "llarp/net/ip_packet.hpp"
#include "llarp/util/thread/queue.hpp"
#include "platform_stats.hpp"
#include "name_resolver.hpp"
#include "traffic.hpp"


#include <llarp/layers/flow/traffic.hpp>

#include <llarp/config/config.hpp>
#include <llarp/net/ip_range.hpp>
#include <llarp/util/formattable.hpp>
#include <llarp/util/types.hpp>

#include <memory>
#include <optional>
#include <string>
#include <string_view>
#include <vector>

namespace llarp
{
    class RoutePoker;
}

namespace llarp::dns
{
  class Server;
}

namespace llarp::vpn
{
    class NetworkInterface;
}


namespace llarp::layers::flow
{
    class FlowLayer;
    class FlowAddr;
    struct FlowInfo;
}


namespace llarp::layers::platform
{

  class PlatformDriver;

  // handles reading and writing os traffic, like ip packets or whatever embedded lokinet wants to do.
  class OSTraffic_IO_Base
  {
    PlatformAddr _our_addr;
      std::vector<OSTraffic> _recv_queue;
  protected:
    PlatformDriver & _parent;
    std::shared_ptr<vpn::NetworkInterface> _netif;

   public:
    explicit OSTraffic_IO_Base(PlatformDriver &);

      const PlatformAddr &
      our_platform_addr() const;
    

      void
      attach(std::shared_ptr<vpn::NetworkInterface> netif);
      
    virtual std::vector<OSTraffic>
    read_platform_traffic() const;

    virtual void
    write_platform_traffic(std::vector<OSTraffic>&& traff) const;
  };

  class PlatformDriver_pimpl;

  class PlatformDriver
  {
   protected:

    friend class OSTraffic_IO_Base;
      
    std::unique_ptr<PlatformDriver_pimpl> _impl;
    flow::FlowLayer & _flow_layer;
      
    std::shared_ptr<EventLoopWakeup> _os_send;
    std::shared_ptr<EventLoopWakeup> _os_recv;

    OSTraffic_IO_Base _os_io;
    AddrMapper _addr_mapper;
      
    std::shared_ptr<llarp::dns::Server> _dns;


    void
    on_before_loop_ready();

    bool
    on_loop_ready();

    void
    after_loop_ready();

   public:

      PlatformDriver(const PlatformDriver &) = delete;
      PlatformDriver(PlatformDriver &&) = delete;

      
    PlatformDriver(
        PlatformDriver_pimpl* impl,
        flow::FlowLayer & flow_layer,
        std::shared_ptr<NetworkConfig> netconf,
        std::shared_ptr<DnsConfig> dnsconf);
      
    virtual void
    got_os_traffic(std::vector<OSTraffic> pkts);

    std::shared_ptr<NameResolverJob_Base>
    make_resolver_job_for(UnresolvedName_t name) const;

    void
    map_remote(
        std::string name,
        std::string auth,
        std::vector<IPRange> ranges,
        std::function<void(std::optional<flow::FlowInfo>, std::string)> result_handler);

    void
    map_exit(
        std::string name,
        std::string auth,
        std::vector<IPRange> ranges,
        std::function<void(bool, std::string)> result_handler);

    PlatformStats
    current_stats() const;
  };
}  // namespace llarp::layers::platform

namespace llarp
{
  template <>
  inline constexpr bool IsToStringFormattable<layers::platform::PlatformAddr> = true;
}
```


-------------------------------

Updated on 2023-02-01 at 19:28:30 +0000
